/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include <gnome.h>
#include <libgnome/libgnome.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include <libgnomeui/gnome-stock.h>
#include "pan.h"
#include "xpm/book_open.xpm"
#include "xpm/book_closed.xpm"
#include "xpm/mini_page.xpm"
#include "xpm/greencheck.xpm"
#include "xpm/bluecheck.xpm"
#include "xpm/x.xpm"
#include "xpm/binary.xpm"
#include "xpm/binary_complete.xpm"
#include "xpm/binary_incomplete.xpm"

// Global Variables
static GtkCTreeNode *parent;
static GnomePixmap *articlelist_closed_pixmap;
static GnomePixmap *articlelist_open_pixmap;
static GnomePixmap *decode_queue_pixmap;
static GnomePixmap *download_queue_pixmap;
static GnomePixmap *article_downloaded_pixmap;
static GnomePixmap *mp_complete_pixmap;
static GnomePixmap *mp_incomplete_pixmap;

article_data *current_article = NULL;
GtkWidget *article_clist_menu;

void
articlelist_update_node(article_data * adata)
{
    GtkCTreeNode *node = NULL;

    g_assert(adata && adata->message_id);

    if (adata->node) {
        node = adata->node;
    } else {
		node = gtk_ctree_find_by_row_data(GTK_CTREE(Pan.article_ctree), NULL, adata);
	}

    if (node) {
        if (adata->state & STATE_DOWNLOAD_QUEUED) {
            gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node, adata->subject, 5,
                                    download_queue_pixmap->pixmap, download_queue_pixmap->mask,
                                    download_queue_pixmap->pixmap, download_queue_pixmap->mask,
                                    GTK_CTREE_ROW(node)->is_leaf,
                                    GTK_CTREE_ROW(node)->expanded);

        } else if (adata->state & STATE_DECODE_QUEUED) {
            gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node, adata->subject, 5,
                                    decode_queue_pixmap->pixmap, decode_queue_pixmap->mask,
                                    decode_queue_pixmap->pixmap, decode_queue_pixmap->mask,
                                    GTK_CTREE_ROW(node)->is_leaf,
                                    GTK_CTREE_ROW(node)->expanded);
        } else if (adata->state & STATE_ERROR) {
            gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node, adata->subject, 5,
                                    Pan.error_pixmap->pixmap, Pan.error_pixmap->mask,
                                    Pan.error_pixmap->pixmap, Pan.error_pixmap->mask,
                                    GTK_CTREE_ROW(node)->is_leaf,
                                    GTK_CTREE_ROW(node)->expanded);
        } else if (adata->state & STATE_DECODED) {
            gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
                                    adata->subject, 5, Pan.binary_decoded_pixmap->pixmap,
                                    Pan.binary_decoded_pixmap->mask, Pan.binary_decoded_pixmap->pixmap,
                                    Pan.binary_decoded_pixmap->mask, GTK_CTREE_ROW(node)->is_leaf,
                                    GTK_CTREE_ROW(node)->expanded);
        } else if (adata->state & STATE_READ) {
            gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
                                    adata->subject, 5, Pan.article_read_pixmap->pixmap,
                                    Pan.article_read_pixmap->mask, Pan.article_read_pixmap->pixmap,
                                    Pan.article_read_pixmap->mask, GTK_CTREE_ROW(node)->is_leaf,
                                    GTK_CTREE_ROW(node)->expanded);
        } else {
            gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node, adata->subject, 5,
                                    NULL, NULL, NULL, NULL, GTK_CTREE_ROW(node)->is_leaf,
                                    GTK_CTREE_ROW(node)->expanded);
        }
 		/* gtk_widget_queue_draw(GTK_WIDGET(Pan.article_ctree)); */
    }
}

void
articlelist_get_all(void)
{
    if (Pan.displayed_group)
        queue_add_articlelist_download(Pan.displayed_group, QUEUE_ARTICLELIST_DOWNLOAD_ALL |
                                 QUEUE_FOREGROUND);
}

void
articlelist_get_new(void)
{
    if (Pan.displayed_group)
        queue_add_articlelist_download(Pan.displayed_group, QUEUE_ARTICLELIST_DOWNLOAD_NEW |
                                 QUEUE_FOREGROUND);
}

void
articlelist_free(GSList * alist)
{
    GSList *p;
    article_data *adata;

    for (p = alist; p != NULL; p = p->next) {
        adata = p->data;

        g_assert(adata);

        if (adata->threads)
            articlelist_free(adata->threads);
        article_free(adata);
        p->data = NULL;
    }
    g_slist_free(alist);
}

void 
process_selections(gint type)
{
    GList *list;
    article_data *adata;

    for (list = GTK_CLIST(Pan.article_ctree)->selection; list; list = list->next) {
        adata = (article_data *)
               gtk_ctree_node_get_row_data(GTK_CTREE(Pan.article_ctree), list->data);

        if (adata == NULL)
            continue;

        if (type == 2) {
            if (adata->parts == 1)
                queue_add_article_download(adata, QUEUE_ARTICLE_DECODE | QUEUE_BACKGROUND);
            else if ((adata->parts >= 2) && (adata->threads))
                queue_add_multipart_download(adata, QUEUE_MULTIPART_DECODE | QUEUE_BACKGROUND);
            else
                continue;
            adata->state |= STATE_DECODE_QUEUED;
			/* Calling article_update here is too slow */
			articlelist_update_node(adata);
        }
    }
    gtk_ctree_unselect_recursive(GTK_CTREE(Pan.article_ctree), parent);
}

void
download_selected_articles(void)
{
    process_selections(0);
}

void
decode_selected_articles(void)
{
    process_selections(2);
}

GtkCTreeNode *
add_article_to_ctree(GtkCTreeNode * parent, article_data * adata)
{
    gchar *text[4];
    GtkCTreeNode *node;

    if (adata->subject != NULL)
        text[0] = adata->subject;
    text[1] = g_strdup_printf("%d", adata->linecount);
    if (adata->date != 0)
        text[2] = g_strdup(ctime((const time_t *) &adata->date));
    if (adata->author != NULL)
        text[3] = adata->author;

    if (adata->state & STATE_ERROR) {
        node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent, NULL, text, 5,
                                     Pan.error_pixmap->pixmap, Pan.error_pixmap->mask,
                                     Pan.error_pixmap->pixmap, Pan.error_pixmap->mask,
                                     FALSE, FALSE);
    } else if (adata->state & STATE_DECODED) {
        node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent, NULL, text, 5,
                                     Pan.binary_decoded_pixmap->pixmap, Pan.binary_decoded_pixmap->mask,
                                     Pan.binary_decoded_pixmap->pixmap, Pan.binary_decoded_pixmap->mask,
                                     FALSE, FALSE);
    } else if (adata->state & STATE_READ) {
        node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent, NULL, text, 5,
                                     Pan.article_read_pixmap->pixmap, Pan.article_read_pixmap->mask,
                                     Pan.article_read_pixmap->pixmap, Pan.article_read_pixmap->mask,
                                     FALSE, FALSE);
    } else if (adata->state & STATE_DOWNLOAD_QUEUED) {
        node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent, NULL, text, 5,
                                     download_queue_pixmap->pixmap, download_queue_pixmap->mask,
                                     download_queue_pixmap->pixmap, download_queue_pixmap->mask,
                                     FALSE, FALSE);
    } else if (adata->state & STATE_DECODE_QUEUED) {
        node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent, NULL, text, 5,
                                     decode_queue_pixmap->pixmap, decode_queue_pixmap->mask,
                                     decode_queue_pixmap->pixmap, decode_queue_pixmap->mask,
                                     FALSE, FALSE);
    } else if (adata->state & STATE_MULTIPART_ALL) {
        node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent, NULL, text, 5,
                                     mp_complete_pixmap->pixmap, mp_complete_pixmap->mask,
                                     mp_complete_pixmap->pixmap, mp_complete_pixmap->mask,
                                     FALSE, FALSE);
    } else if (adata->state & STATE_MULTIPART_SOME) {
        node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent, NULL, text, 5,
                                     mp_incomplete_pixmap->pixmap, mp_incomplete_pixmap->mask,
                                     mp_incomplete_pixmap->pixmap, mp_incomplete_pixmap->mask,
                                     FALSE, FALSE);
    } else {
        node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent, NULL, text, 5,
                                     NULL, NULL, NULL, NULL, FALSE, FALSE);
    }
    gtk_ctree_node_set_row_data(GTK_CTREE(Pan.article_ctree), node, (article_data *) adata);
    adata->node = node;
    return(node);
}

static int
build_article_ctree_recursive(GtkCTreeNode * parent, GSList * list)
{
    GSList *pos;
    article_data *data;
    GtkCTreeNode *sibling;
    int count = 0;
    pos = list;

    while (pos) {
        data = (article_data *) pos->data;
        pos = pos->next;
        sibling = add_article_to_ctree(parent, data);
        ++count;
        if (data->threads) {
            count = count + build_article_ctree_recursive(sibling, data->threads);
        }
    }
    return(count);
}

void
article_ctree_clean(void)
{
    gtk_clist_clear(GTK_CLIST(Pan.article_ctree));
    if (Pan.displayed_group) {
        clean_group(Pan.displayed_group);
        Pan.displayed_group = NULL;
    }
}

void
article_ctree_update(group_data * gdata)
{
    gchar *text[4];
    GtkCTreeNode *parent;

    text[0] = g_strdup(gdata->name);
    text[1] = text[2] = text[3] = NULL;

	gui_set_status(_("Updating Article Display"));
    pan_lock();
    gtk_clist_clear(GTK_CLIST(Pan.article_ctree));
    gtk_clist_freeze(GTK_CLIST(Pan.article_ctree));
    parent = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), NULL, NULL, text, 5,
                                   articlelist_closed_pixmap->pixmap, articlelist_closed_pixmap->mask,
                                   articlelist_open_pixmap->pixmap, articlelist_open_pixmap->mask, FALSE, TRUE);

    build_article_ctree_recursive(parent, gdata->article_list);

    gtk_clist_thaw(GTK_CLIST(Pan.article_ctree));

    if (Pan.displayed_group && (Pan.displayed_group != gdata)) {
        clean_group(Pan.displayed_group);
    }
    Pan.displayed_group = gdata;
    pan_unlock();
}

int
multipart_sortfunc(article_data * data1, article_data * data2)
{
    if (data1->part > data2->part)
        return(1);
    else
        return(0);
}

int
mp_strcmp(gchar * mp, gchar * ph)
{
    gint i = 0;
    gint j = 0;
    gint k = 0;
    gint l = 0;

    while ((*(mp + i) != '\0') && (*(ph + j) != '\0')) {
        if (isdigit((int) *(ph + j)) && isdigit((int) *(mp + i))) {
            for (k = 0; isdigit((int) *(ph + j + k)); ++k);
            j += k;
            for (l = 0; isdigit((int) *(mp + i + l)); ++l);
            i += l;
        } else if (*(mp + i) == *(ph + j)) {
            i++;
            j++;
        } else
            return 1;
    }
    //if ( j - k == i - l )
    return 0;
    //else
    //      return 1;
}

GSList *
thread_article_by_multipart(GSList * list, article_data * adata)
{
    GSList *p;
    article_data *data;

    g_assert(list && adata);

    p = list;
    while (p) {
        g_assert(p->data);
        data = p->data;
        p = p->next;
        if ((data->parts >= 2) && (data->parts == adata->parts) && (data->part < adata->part)) {
            if (!mp_strcmp(data->subject, adata->subject)) {
                list = g_slist_remove(list, adata);
                if (adata->threads) {
                    data->threads = g_slist_concat(data->threads, adata->threads);
                    data->nthreads = data->nthreads + adata->nthreads;
                    adata->threads = NULL;
                    adata->nthreads = 0;
                }
                data->threads = g_slist_insert_sorted(data->threads, adata,
                                                      (GCompareFunc) multipart_sortfunc);
                data->nthreads++;
                return(list);
            }
        }
    }
    return(list);
}

GSList *
thread_list_by_multipart(GSList * list, group_data * gdata)
{
    article_data *adata;
    GSList *p;
    gfloat count, total;

    g_assert(list);

    count = 0;
    total = gdata->articles * 3;

    p = list;

    while (p) {
        gui_set_progress(count++ / total);
        adata = (article_data *) p->data;
        p = p->next;
        if (adata->parts >= 2) {
            // Found a Multipart, thread it
            list = thread_article_by_multipart(list, adata);
        }
    }
    for (p = list; p != NULL; p = p->next) {
        adata = (article_data *) p->data;
        if (adata->parts >= 2) {
            if ((adata->nthreads == (adata->parts - 1)) || (adata->nthreads == adata->parts))
                adata->state = STATE_MULTIPART_ALL;
            else
                adata->state = STATE_MULTIPART_SOME;
        }
    }
    return list;
}

gint
thread_article_by_reference(GSList * list, article_data * data)
{
    GSList *pos;
    article_data *adata;

    pos = list;

    while (pos) {
        adata = (article_data *) pos->data;
        pos = pos->next;
        if (adata->references && (strcmp(data->message_id, adata->message_id))) {
            if (!strcmp(strrchr(data->references, '<'), adata->message_id)) {
                // we found our home! Thread it here
                adata->threads = g_slist_prepend(adata->threads, data);
                adata->nthreads++;
                return 1;
            }
            if (adata->threads) {
                if (thread_article_by_reference(adata->threads, data))
                    return 1;
            }
        }
    }
    return 0;
}

GSList *
thread_thread_by_reference(GSList * list)
{
    article_data *adata;
    GSList *tmp;
    int rc;

    tmp = list;
    while (tmp) {
        adata = (article_data *) tmp->data;
        tmp = tmp->next;
        if (adata->references && adata->references[0] == '<') {
            rc = thread_article_by_reference(list, adata);
            if (rc) {
                list = g_slist_remove(list, adata);
            }
        }
    }
    return list;
}

GSList *
thread_list_by_reference(GSList * list, group_data * gdata)
{
    article_data *adata;
    GSList *tmp;
    gfloat count, total;

    count = (gdata->articles) * 2;
    total = (gdata->articles) * 3;
    tmp = list;

    while (tmp) {
        gui_set_progress(count++ / total);

        adata = (article_data *) tmp->data;
        tmp = tmp->next;
        if (adata->threads) {
            adata->threads = thread_thread_by_reference(adata->threads);
        }
    }
    gui_set_progress(1);
    return list;
}

gint
thread_article_by_subject(GSList * list, article_data * data)
{
    GSList *tmp;
    article_data *adata, *min = NULL;

    tmp = list;

    while (tmp) {
        adata = (article_data *) tmp->data;
        tmp = tmp->next;
        if (adata != data) {
            if (!strcmp(adata->subject, data->subject + 4)) {
                // Found  "Re: name" and "name"
                adata->threads = g_slist_prepend(adata->threads, data);
                adata->nthreads++;
                return 1;
            } else if (!strcasecmp(adata->subject, data->subject)) {
                if (adata->threads) {
                    adata->threads = g_slist_prepend(adata->threads, data);
                    adata->nthreads++;
                    return 1;
                } else {
                    if (min == NULL) {
                        min = (adata->date < data->date) ? adata : NULL;
                    } else if (adata->date < min->date) {
                        min = adata;
                    }
                }
            }
        }
    }
    if (min) {
        min->threads = g_slist_prepend(min->threads, data);
        min->nthreads++;
        return 1;
    } else
        return 0;
}

GSList *
thread_list_by_subject(GSList * list, group_data * gdata)
{
    article_data *adata;
    GSList *pos, *last;
    gint rc;
    gfloat count, total;

    g_assert(list);

    count = gdata->articles;
    total = gdata->articles * 3;

    pos = list;

    while (pos) {
        gui_set_progress(count++ / total);
        adata = (article_data *) pos->data;
        last = pos;
        pos = pos->next;
        if (!strncasecmp("re:", adata->subject, 3)) {
            rc = thread_article_by_subject(list, adata);
            if (rc) {
                list = g_slist_remove(list, adata);
            }
        }
    }
    return list;
}

static void
article_ctree_button_press(GtkWidget * widget, GdkEventButton * bevent)
{
    gint row;
    gint column;
    article_data *adata;

    if (bevent->button != 1)
        gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");

    if ((bevent->button) == 1 && (bevent->type == GDK_2BUTTON_PRESS)) {
        gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");
        gtk_clist_get_selection_info(GTK_CLIST(widget), bevent->x, bevent->y,
                                     &row, &column);
        adata = (article_data *) gtk_clist_get_row_data(GTK_CLIST(widget), row);
        if (adata) {
            current_article = adata;
            if (adata->body == NULL) {
                queue_add_article_download(adata, QUEUE_ARTICLE_DOWNLOAD | QUEUE_FOREGROUND);
                adata->state |= STATE_DOWNLOAD_QUEUED;
                articlelist_update_node(adata);
            } else {
                update_text(adata);
            }
        }
    } else if (bevent->button == 3) {
        gtk_menu_popup(GTK_MENU(article_clist_menu), NULL, NULL, NULL,
                       NULL, bevent->button, bevent->time);
    }
}

void
create_article_ctree_menu(void)
{
    GtkWidget *tempitem;

    article_clist_menu = gtk_menu_new();

    tempitem = gtk_menu_item_new_with_label("Decode Selected");
    gtk_menu_append(GTK_MENU(article_clist_menu), tempitem);
    gtk_signal_connect_object(GTK_OBJECT(tempitem), "activate", GTK_SIGNAL_FUNC(decode_selected_articles), NULL);
    gtk_widget_show(tempitem);
}

int
compare_subject_ascending(article_data * data1, article_data * data2)
{
    return(strcasecmp(data1->subject, data2->subject));
}

int
compare_subject_descending(article_data * data1, article_data * data2)
{
    return(strcasecmp(data2->subject, data1->subject));
}

int
compare_lines_ascending(article_data * data1, article_data * data2)
{
    if (data1->linecount > data2->linecount)
        return(1);
    else
        return(-1);
}

int
compare_lines_descending(article_data * data1, article_data * data2)
{
    if (data2->linecount > data1->linecount)
        return(1);
    else
        return(-1);
}

int
compare_author_ascending(article_data * data1, article_data * data2)
{
    return(strcasecmp(data1->author, data2->author));
}

int
compare_author_descending(article_data * data1, article_data * data2)
{
    return(strcasecmp(data2->author, data1->author));
}

int
compare_date_ascending(article_data * data1, article_data * data2)
{
    if (data1->date > data2->date)
        return(1);
    else
        return(-1);
}

int
compare_date_descending(article_data * data1, article_data * data2)
{
    if (data2->date > data1->date)
        return(1);
    else
        return(-1);
}

static void
click_articlelist_column(GtkCList * clist, gint n)
{
    static gint subject = 1;
    static gint lines = 1;
    static gint date = 1;
    static gint author = 1;

    if (Pan.displayed_group == NULL)
        return;

    switch (n) {
    case 0:
        if (subject) {
            Pan.displayed_group->article_list = g_slist_sort(Pan.displayed_group->article_list,
                                                             (GCompareFunc) compare_subject_ascending);
            subject = 0;
        } else {
            Pan.displayed_group->article_list = g_slist_sort(Pan.displayed_group->article_list,
                                                             (GCompareFunc) compare_subject_descending);
            subject = 1;
        }
        break;
    case 1:
        if (lines) {
            Pan.displayed_group->article_list = g_slist_sort(Pan.displayed_group->article_list,
                                                             (GCompareFunc) compare_lines_ascending);
            lines = 0;
        } else {
            Pan.displayed_group->article_list = g_slist_sort(Pan.displayed_group->article_list,
                                                             (GCompareFunc) compare_lines_descending);
            lines = 1;
        }
        break;
    case 2:
        if (date) {
            Pan.displayed_group->article_list = g_slist_sort(Pan.displayed_group->article_list,
                                                             (GCompareFunc) compare_date_ascending);
            date = 0;
        } else {
            Pan.displayed_group->article_list = g_slist_sort(Pan.displayed_group->article_list,
                                                             (GCompareFunc) compare_date_descending);
            date = 1;
        }
        break;
    case 3:
        if (author) {
            Pan.displayed_group->article_list = g_slist_sort(Pan.displayed_group->article_list,
                                                             (GCompareFunc) compare_author_ascending);
            author = 0;
        } else {
            Pan.displayed_group->article_list = g_slist_sort(Pan.displayed_group->article_list,
                                                             (GCompareFunc) compare_author_descending);
            author = 1;
        }
        break;
    default:
        g_warning("Reached default in click_articlelist_column()\n");
        return;
    }
    article_ctree_update(Pan.displayed_group);
}

GtkWidget *
create_articlelist_ctree()
{
    GtkWidget *s_window;

    static char *titles[] =
    {
        _("Subject"),
        _("Lines"),
        _("Date"),
        _("Author"),
    };

    articlelist_closed_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(book_closed_xpm);
    articlelist_open_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(book_open_xpm);
    Pan.article_read_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(mini_page_xpm);
    decode_queue_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(bluecheck_xpm);
    download_queue_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(greencheck_xpm);
    article_downloaded_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(mini_page_xpm);
    Pan.error_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(x_xpm);
    Pan.binary_decoded_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(binary_xpm);
    mp_complete_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(binary_complete_xpm);
    mp_incomplete_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(binary_incomplete_xpm);

    Pan.article_ctree = gtk_ctree_new_with_titles(4, 0, titles);
    gtk_ctree_set_line_style(GTK_CTREE(Pan.article_ctree), GTK_CTREE_LINES_DOTTED);
    gtk_widget_show(Pan.article_ctree);
    s_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(s_window),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(s_window), Pan.article_ctree);
    gtk_widget_show(s_window);

    gtk_signal_connect(GTK_OBJECT(Pan.article_ctree), "button_press_event",
                       GTK_SIGNAL_FUNC(article_ctree_button_press), NULL);

    gtk_signal_connect(GTK_OBJECT(Pan.article_ctree), "click_column",
                       GTK_SIGNAL_FUNC(click_articlelist_column), NULL);

    gtk_clist_set_selection_mode(GTK_CLIST(Pan.article_ctree), GTK_SELECTION_EXTENDED);
    gtk_clist_set_column_width(GTK_CLIST(Pan.article_ctree), 0, 400);
    gtk_clist_set_column_width(GTK_CLIST(Pan.article_ctree), 2, 200);

    // Create the Popup Menu While we are here
    create_article_ctree_menu();

    return s_window;
}

/***************************************************************
* articlelist_load 
****************************************************************/

void
articlelist_load(queue_data * qdata)
{
    group_data *gdata;
	GSList *alist;
    
    gdata = qdata->gdata;

    gui_set_status(_("Loading Article Headers"));
    if ((alist = db_load_articles(gdata)) == NULL) {
        pan_lock();
        choose_download_dialog(gdata);
        pan_unlock();
		goto articlelist_load_cleanup;
    }
    group_clist_update_row(gdata);
   	gui_set_info_va(_("Loaded %d article headers"), gdata->articles); 
	gui_set_status(_("Threading Article List"));

	alist = thread_list_by_multipart(alist, gdata);
	alist = thread_list_by_subject(alist, gdata);
    alist = thread_list_by_reference(alist, gdata);
    gdata->article_list = alist;
    
	gui_set_status(_("Displaying article list"));
    article_ctree_update((group_data *) gdata);
	articlelist_load_cleanup:
    g_free(qdata);
	gui_clean();
}
