/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include "pan.h"
#include "xpm/disk.xpm"

extern int errno;

extern server_data *active_server;
extern GtkWidget *clist_groups;
extern GSList *current_article_list;

//Globals
static gint group_mode = GROUP_ALL;
GtkWidget *groups_clist_menu;
static GnomePixmap *disk_pixmap = NULL;

//Callback function
static void
click_groups_column_cb(GtkCList * clist, gint n)
{
    if (n)
        return;
    switch(group_mode) {
	case GROUP_SUBSCRIBED:
        group_mode = GROUP_NEW;
		break;
	case GROUP_NEW:
        group_mode = GROUP_ALL;
		break;
	case GROUP_ALL:
        group_mode = GROUP_SUBSCRIBED;
		break;
	}
    group_clist_update();
}

void
download_selected_groups_cb(void)
{
    GList *list;
    group_data *data;

    list = GTK_CLIST(Pan.group_clist)->selection;
    while (list) {
        data = (group_data *)
               gtk_clist_get_row_data(GTK_CLIST(Pan.group_clist), GPOINTER_TO_INT(list->data));
        queue_add_articlelist_download(data, QUEUE_ARTICLELIST_DOWNLOAD_ALL | QUEUE_BACKGROUND);
        list = list->next;
    }
    clist_unselect_all(Pan.group_clist);
}

void
download_selected_groups_new_cb(void)
{
    GList *list;
    group_data *data;

    list = GTK_CLIST(Pan.group_clist)->selection;
    while (list) {
        data = (group_data *) gtk_clist_get_row_data(GTK_CLIST(Pan.group_clist), GPOINTER_TO_INT(list->data));
        queue_add_articlelist_download(data, QUEUE_ARTICLELIST_DOWNLOAD_NEW | QUEUE_BACKGROUND);
        list = list->next;
    }
    clist_unselect_all(Pan.group_clist);
}

gint
compare_group_data(group_data * data0, group_data * data1)
{
    return(strcmp(data0->name, data1->name));
}

void
group_clist_update_row(group_data *gdata)
{
    gint row;
    gchar *name = NULL;

    pan_lock();
	row = gtk_clist_find_row_from_data(GTK_CLIST(Pan.group_clist), gdata);

    if (gdata->articles) {
        name = g_strdup_printf("%s (%d)", gdata->name, gdata->articles);
    } else {
        name = g_strdup(gdata->name);
    }
    
    if (gdata->flags & GROUP_ONDISK) {
        gtk_clist_set_pixtext(GTK_CLIST(Pan.group_clist), row, 0, name, 5,
                              disk_pixmap->pixmap, disk_pixmap->mask);
    } else {
        gtk_clist_set_text(GTK_CLIST(Pan.group_clist), row, 0, name);
    }
    g_free(name);
    gtk_widget_queue_draw(GTK_WIDGET(Pan.group_clist));
    pan_unlock();
}

void
add_group_to_clist(group_data *gdata)
{
    gint row;
    gchar *text[3];
    GtkCListRow *clistrow;
    gchar *name = NULL;

    if (gdata->articles) {
        name = g_strdup_printf("%s (%d)", gdata->name, gdata->articles);
    } else {
		name = g_strdup(gdata->name);
    }
    text[0] = name;
    text[1] = NULL;

    row = gtk_clist_append(GTK_CLIST(Pan.group_clist), text);
    if (gdata->flags & GROUP_ONDISK) {
        gtk_clist_set_pixtext(GTK_CLIST(Pan.group_clist), row, 0, name, 5,
                              disk_pixmap->pixmap, disk_pixmap->mask);
    }
    g_free(name);

    /* The following hack is *MUCH* faster than this:
     * gtk_clist_set_row_data(GTK_CLIST(Pan.group_clist), row, gdata);
     * Let's hope the gtk structure doesn't change or something */
    clistrow = (GtkCListRow *) GTK_CLIST(Pan.group_clist)->row_list_end->data;
    clistrow->data = gdata;

} 

void
add_subscribed_group_to_clist(group_data *gdata)
{
    if (gdata->flags & GROUP_SUBSCRIBED)
        add_group_to_clist(gdata);
}

void
add_new_group_to_clist(group_data *gdata)
{
    if (gdata->flags & GROUP_NEW)
        add_group_to_clist(gdata);
}

void
group_dialog_clicked_cb(GnomeDialog * dialog, gint num)
{
    switch (num) {
    case 0:         /* OK button */
        queue_add_grouplist_download();
    default:
        gtk_widget_destroy(GTK_WIDGET(dialog));
        break;
    }
}

void
group_free(group_data * data)
{
    g_return_if_fail(data->name && data);

    g_free(data->name);
    g_free(data);
}

void
grouplist_free(GSList * list)
{
    if (list) {
        g_slist_foreach(list, (GFunc) group_free, NULL);
        g_slist_free(list);
        list = NULL;
    }
}

void
get_all_groups(void)
{
    queue_add_grouplist_download();
}

static void
save_group(group_data * gdata, FILE * fp)
{
    fprintf(fp, "%d\n%s\n", gdata->flags, gdata->name);
}

void
grouplist_save(void)
{
    FILE *fp;
    char path[PATH_MAX];

    g_return_if_fail(Pan.grouplist);

    g_snprintf(path, 256, "/%s/%s/active", Pan.props->data_dir, Pan.active_server->name);

    if ((fp = fopen(path, "w")) != NULL) {
        g_slist_foreach(Pan.grouplist, (GFunc) save_group, fp);
        fclose(fp);
    } else {
    	pan_error_dialog(_("We couldn't save group list to %s:\n%s"), path, strerror(errno));
	}
}

void
grouplist_load(void)
{
    GnomeDialog *dialog;
    GtkWidget *label;
    group_data *gdata;
    GSList *prev_list = NULL;
    GSList *temp_list;
    int flags = 0;
    char *p = NULL;
    FILE *group_file;
    char path[PATH_MAX];

    g_return_if_fail(Pan.active_server && Pan.active_server->name);

    g_snprintf(path, 256, "/%s/%s/active", Pan.props->data_dir, Pan.active_server->name);

    if ((group_file = fopen(path, "r")) == NULL) {
        dialog = (GnomeDialog *) gnome_dialog_new(_("No Group List"),
                                                  GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
        label = gtk_label_new(_("We don't have a group list for the active server.\nShall we get one?\n"));
        gtk_container_add(GTK_CONTAINER(dialog->vbox), label);
        gtk_signal_connect(GTK_OBJECT(dialog), "clicked",
                           GTK_SIGNAL_FUNC(group_dialog_clicked_cb), NULL);
        gtk_widget_show(label);
        gtk_widget_show(GTK_WIDGET(dialog));
        return;
    }
    
    while (1) {
        fscanf(group_file, "%d\n", &flags);
        if (feof(group_file))
           break;
        p = fgetline(group_file);
        
        gdata = (group_data *) g_malloc(sizeof(group_data));
        memset(gdata, 0, sizeof(group_data));
        gdata->name = p;
        gdata->flags = flags;
        
        temp_list = g_slist_alloc();
        if (Pan.grouplist == NULL)
            Pan.grouplist = temp_list;
        temp_list->data = gdata;
        g_slist_concat(prev_list, temp_list);
        prev_list = temp_list;                
    }
    fclose(group_file);
}


void
choose_download_dialog_clicked_cb(GnomeDialog * dialog, gint button_number, gpointer gdata)
{
    switch (button_number) {
    case 0:         /* All Headers */
        queue_add_articlelist_download(gdata, QUEUE_ARTICLELIST_DOWNLOAD_ALL
                                 | QUEUE_FOREGROUND);
        break;
    case 1:         /* New Headers */
        queue_add_articlelist_download(gdata, QUEUE_ARTICLELIST_DOWNLOAD_SAMPLE
                                 | QUEUE_FOREGROUND);
        break;
    default:
        /* Cancel */
        break;
    };
    gtk_widget_destroy(GTK_WIDGET(dialog));
}

void
choose_download_dialog(group_data * gdata)
{
    GtkWidget *dialog;
    GtkWidget *label;
    gchar *temp;

    dialog = gnome_dialog_new(_("Select Header Download Option"), _("Download All Headers"),
                              _("Sample Some Headers"), GNOME_STOCK_BUTTON_CANCEL,
                              NULL);

    gtk_signal_connect(GTK_OBJECT(dialog), "clicked",
                       (GtkSignalFunc) choose_download_dialog_clicked_cb, gdata);

    temp = g_strdup_printf(_("Select an option for group: %s"), gdata->name);
    label = gtk_label_new(temp);
    g_free(temp);

    gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);
    gtk_widget_show(label);

    gtk_widget_show(dialog);
}

void
select_group_row(GtkWidget * widget, gint r, gint c, GdkEventButton * bevent)
{
    group_data *gdata;

    if (bevent) {
        if ((bevent->button) == 1 && (bevent->type == GDK_2BUTTON_PRESS)) {
            gdata = (group_data *) gtk_clist_get_row_data(GTK_CLIST(widget), r);
            if (group_index_exists(gdata))
                queue_add_articlelist_download(gdata, QUEUE_ARTICLELIST_LOAD | QUEUE_FOREGROUND);
            else
                choose_download_dialog(gdata);
        }
    }
}

static void
groups_clist_button_press(GtkWidget * widget, GdkEventButton * bevent)
{
    if (bevent->button != 1)
        gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");

    if (bevent->button == 3)
        gtk_menu_popup(GTK_MENU(groups_clist_menu), NULL, NULL, NULL,
                       NULL, bevent->button, bevent->time);
}

void
create_groups_clist_menu(void)
{
    GtkWidget *tempitem;

    groups_clist_menu = gtk_menu_new();

    /* menu items to our menu.  Normally, we'd also catch the "clicked"
     * signal on each of the menu items and setup a callback for it,
     * but it's omitted here to save space. */

    tempitem = gtk_menu_item_new_with_label("Get All Headers from Selected");
    gtk_menu_append(GTK_MENU(groups_clist_menu), tempitem);
    gtk_signal_connect_object(GTK_OBJECT(tempitem), "activate", GTK_SIGNAL_FUNC(download_selected_groups_cb), NULL);
    gtk_widget_show(tempitem);
    tempitem = gtk_menu_item_new_with_label("Get New Headers from Selected");
    gtk_menu_append(GTK_MENU(groups_clist_menu), tempitem);
    gtk_signal_connect_object(GTK_OBJECT(tempitem), "activate", GTK_SIGNAL_FUNC(download_selected_groups_new_cb), NULL);
    gtk_widget_show(tempitem);   
}

GtkWidget *
create_groups_clist(void)
{
    GtkWidget *temp;

    static char *titles[] =
    {
        "All Groups",        
    };


    Pan.group_clist = gtk_clist_new_with_titles(1, titles);
    gtk_widget_show(Pan.group_clist);
    temp = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(temp),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(temp), Pan.group_clist);
    gtk_widget_show(temp);

    gtk_signal_connect(GTK_OBJECT(Pan.group_clist),
                       "select_row",
                       (GtkSignalFunc) select_group_row,
                       NULL);

    gtk_signal_connect(GTK_OBJECT(Pan.group_clist),
                       "unselect_row",
                       (GtkSignalFunc) select_group_row,
                       NULL);

    gtk_signal_connect(GTK_OBJECT(Pan.group_clist),
                       "button_press_event",
                       (GtkSignalFunc) groups_clist_button_press,
                       NULL);

    gtk_clist_set_column_width(GTK_CLIST(Pan.group_clist), 0, 200);

    gtk_clist_set_selection_mode(GTK_CLIST(Pan.group_clist), GTK_SELECTION_EXTENDED);

    gtk_clist_set_column_justification(GTK_CLIST(Pan.group_clist), 0, GTK_JUSTIFY_LEFT);
    gtk_clist_set_column_justification(GTK_CLIST(Pan.group_clist), 1, GTK_JUSTIFY_LEFT);

    gtk_signal_connect(GTK_OBJECT(Pan.group_clist), "click_column",
                       GTK_SIGNAL_FUNC(click_groups_column_cb), NULL);

    /* Create the Popup Menu while we are here. */
    create_groups_clist_menu();
    /* Lets have the Pixmap ready too */
    disk_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(disk_xpm);

    return temp;
}

void
group_clist_update()
{
    if (Pan.grouplist == NULL) {
        grouplist_load();
    }
    pan_lock();
	gtk_clist_clear(GTK_CLIST(Pan.group_clist));
    switch (group_mode) {
    // All Groups
    case GROUP_ALL:
        gtk_clist_set_column_title(GTK_CLIST(Pan.group_clist), 0, "All Groups");
        gtk_clist_freeze(GTK_CLIST(Pan.group_clist));
        g_slist_foreach(Pan.grouplist, (GFunc) add_group_to_clist, NULL);
        gtk_clist_thaw(GTK_CLIST(Pan.group_clist));
        break;
        // Subscribed Groups
    case GROUP_SUBSCRIBED:
        gtk_clist_set_column_title(GTK_CLIST(Pan.group_clist), 0, "Subscribed Groups");
        gtk_clist_freeze(GTK_CLIST(Pan.group_clist));
        g_slist_foreach(Pan.grouplist, (GFunc) add_subscribed_group_to_clist, NULL);
        gtk_clist_thaw(GTK_CLIST(Pan.group_clist));
        break;
        // New Groups
    case GROUP_NEW:
        gtk_clist_set_column_title(GTK_CLIST(Pan.group_clist), 0, "New Groups");
        gtk_clist_freeze(GTK_CLIST(Pan.group_clist));
        g_slist_foreach(Pan.grouplist, (GFunc) add_new_group_to_clist, NULL);
        gtk_clist_thaw(GTK_CLIST(Pan.group_clist));
        break;
    default:
        g_warning("Reached default in Update_Group_Clist switch\n");
    }
    pan_unlock();
}

void
grep_activated(GtkWidget * group_entry)
{
    gchar *entry_text;
    GSList *pos;
    group_data *gdata;

    g_return_if_fail(Pan.group_clist);

    gtk_clist_clear(GTK_CLIST(Pan.group_clist));
    entry_text = gtk_entry_get_text(GTK_ENTRY(group_entry));
    gtk_clist_freeze(GTK_CLIST(Pan.group_clist));
    pos = Pan.grouplist;
    while (pos != NULL) {
        gdata = pos->data;
        if (((gdata->flags & group_mode) || (group_mode == GROUP_ALL))
				&& strstr(gdata->name, entry_text)) {
            add_group_to_clist(pos->data);
        }
        pos = pos->next;
    }
    gtk_clist_thaw(GTK_CLIST(Pan.group_clist));
}

void
clean_group(group_data * gdata)
{
    articlelist_free(gdata->article_list);
    gdata->article_list = NULL;
}
