/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include "uulib/uudeview.h"
#include "uulib/fptools.h"
#include "pan.h"

static GtkWidget *appbar;
static GtkWidget *progress;

void
gui_clean(void)
{
    pan_lock();
    gnome_appbar_clear_stack(GNOME_APPBAR(appbar));
   	if (progress) {
		gtk_progress_set_activity_mode(GTK_PROGRESS(progress), FALSE);
		progress = NULL;
	}
    gnome_appbar_set_progress(GNOME_APPBAR(appbar), 0);
    pan_unlock();   
}


void
gui_create_appbar(GtkWidget * window)
{
    appbar = gnome_appbar_new(TRUE, TRUE, FALSE);
    gnome_appbar_set_default(GNOME_APPBAR(appbar), _("Pan - Offline"));
    gnome_app_set_statusbar(GNOME_APP(window), appbar);
}

void
gui_set_status(gchar *msg)
{
	pan_lock();
	gnome_appbar_pop(GNOME_APPBAR(appbar));
    gnome_appbar_push(GNOME_APPBAR(appbar), msg);
	pan_unlock();
}

void
gui_set_status_va(const gchar * format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail(format != NULL);

	va_start(args, format);
	msg = g_strdup_vprintf(format, args);
	va_end(args);

	pan_lock();
	gnome_appbar_pop(GNOME_APPBAR(appbar));
    gnome_appbar_push(GNOME_APPBAR(appbar), msg);
	pan_unlock();
	g_free(msg);
}

void
gui_set_progress(gfloat progress)
{
	pan_lock();
	gnome_appbar_set_progress(GNOME_APPBAR(appbar), progress);
	pan_unlock();
}

void
gui_set_status_default(gchar *msg)
{
	pan_lock();
	gnome_appbar_set_default(GNOME_APPBAR(appbar), msg);
	pan_unlock();
}

void
gui_set_progress_activity(gint count)
{
	pan_lock();
	if (progress == NULL) {
		progress = GTK_WIDGET(gnome_appbar_get_progress(GNOME_APPBAR(appbar)));
        gtk_progress_set_activity_mode(GTK_PROGRESS(progress), TRUE);
        gtk_progress_bar_set_discrete_blocks(GTK_PROGRESS_BAR (progress), 10);
        gtk_progress_bar_set_activity_step(GTK_PROGRESS_BAR (progress), 1);
	}
	gtk_progress_set_value(GTK_PROGRESS(progress), (gfloat)(count % 11));
	pan_unlock();
}

/* Infobar */
GtkWidget *infobar;
guint context_id;

GtkWidget *
gui_create_infobar(void)
{
	infobar = gtk_statusbar_new();
	context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(infobar), "blah");
	return (infobar);
}

void
gui_set_info(gchar *msg)
{
	pan_lock();
	gtk_statusbar_pop(GTK_STATUSBAR(infobar), context_id );
	gtk_statusbar_push(GTK_STATUSBAR(infobar), context_id, msg);
	pan_unlock();
}

void
gui_set_info_va(const gchar * format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail(format != NULL);

	va_start(args, format);
	msg = g_strdup_vprintf(format, args);
	va_end(args);

	pan_lock();
	gtk_statusbar_pop(GTK_STATUSBAR(infobar), context_id );
	gtk_statusbar_push(GTK_STATUSBAR(infobar), context_id, msg);
	pan_unlock();
	g_free(msg);
}

/* About box */

void
gui_about(void)
{
    const gchar *authors[] =
    {"e.Messiah@superpimp.org (Matt Eagleson)", NULL};

    GtkWidget *about = gnome_about_new("Pan", VERSION,
                                       _("Copyright e.Messiah (C) 1999"),
                                       authors,
                                       "The Pimp Ass Newsreader (PAN)",
                                       NULL);
    gtk_widget_show(about);
}

void
gui_construct(void)
{
    GtkWidget *window;
	GtkWidget *vbox;
	GtkWidget *group_vbox;
	GtkWidget *group_hbox;
	GtkWidget *a_s_window;
	GtkWidget *text_table;
	GtkWidget *group_entry;
	GtkWidget *group_label;
	GtkWidget *vpaned;
	GtkWidget *hpaned;

	window = gnome_app_new("Pan", _("Pan"));
    gtk_signal_connect(GTK_OBJECT(window), "delete_event", (GtkSignalFunc) pan_shutdown, NULL);
	gtk_widget_set_usize(GTK_WIDGET(window), 640, 400);
    
	gnome_app_set_menus(GNOME_APP(window), GTK_MENU_BAR(menu_create(window)));

	vbox = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(vbox);
    
	group_vbox = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(group_vbox);
    
	group_hbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(group_hbox);
    
	a_s_window = create_articlelist_ctree();
    
	text_table = create_text();
    gtk_widget_show(text_table);
    
	group_entry = gtk_entry_new();
    gtk_widget_show(group_entry);
    gtk_signal_connect(GTK_OBJECT(group_entry), "activate", GTK_SIGNAL_FUNC(grep_activated),
                       group_entry);
    
	group_label = gtk_label_new("Find:");
    gtk_widget_show(group_label);
    
	gtk_box_pack_start(GTK_BOX(group_vbox), create_groups_clist(), TRUE, TRUE, 1);
    gtk_box_pack_start(GTK_BOX(group_vbox), group_hbox, FALSE, FALSE, 1);
    gtk_box_pack_start(GTK_BOX(group_hbox), group_label, FALSE, FALSE, 3);
    gtk_box_pack_start(GTK_BOX(group_hbox), group_entry, TRUE, TRUE, 3);
   
	vpaned = gtk_vpaned_new();
    gtk_widget_show(GTK_WIDGET(vpaned));
    
	hpaned = gtk_hpaned_new();
    gtk_widget_show(GTK_WIDGET(hpaned));
    
	gtk_paned_add1(GTK_PANED(vpaned), hpaned);
    gtk_paned_add1(GTK_PANED(hpaned), group_vbox);
    gtk_paned_add2(GTK_PANED(hpaned), a_s_window);
    gtk_paned_add2(GTK_PANED(vpaned), text_table);
    
	gtk_widget_set_usize(Pan.group_clist, 300, 200);
    gtk_box_pack_start(GTK_BOX(vbox), vpaned, TRUE, TRUE, 0);
    
	gui_create_infobar();
    gtk_box_pack_start(GTK_BOX(vbox), infobar, FALSE, FALSE, 0);
	gtk_widget_show(GTK_WIDGET(infobar));
	
	gnome_app_set_contents(GNOME_APP(window), vbox);
	gui_create_appbar(window);
    gtk_widget_show(window);
}
