/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <libgnomeui/gnome-stock.h>
#include "pan.h"

typedef struct _Props_window Props_window;
struct _Props_window {
    GnomePropertyBox *box;
    GtkWidget *server_clist;
    GtkWidget *edit_profile_dialog;
    GtkWidget *entry1;
    GtkWidget *entry2;
    GtkWidget *entry3;
    GtkWidget *email_entry;
    GtkWidget *organization_entry;
    GtkWidget *fullname_entry;
    GtkWidget *download_entry;
    GtkWidget *temp_entry;
    GtkWidget *data_entry;
};

static Props_window *props_window = NULL;

static Props *new_props;

static void
load_servers(Props * props)
{
    char **sd;
	char *p;
    int i;
    server_data *sdata;
	char buf[256];
	
    g_list_free(props->servers);
    props->servers = NULL;

    if ((p = gnome_config_get_string("/Pan/Servers/Names")) == NULL) {
        return;
    }
    
	sd = g_strsplit(p, ",", 0);
	g_free(p);
	if (sd == NULL) {
        g_error("Error in config file.");
        return;
    }

	for (i = 0; sd[i]; i++) {
 		g_snprintf(buf, 256, "/Pan/%s/", sd[i]);
        if (gnome_config_has_section(buf)) {
           	gnome_config_push_prefix(buf); 
			sdata = (server_data *) g_malloc(sizeof(server_data));
            memset(sdata, 0, sizeof(server_data));
            if ((sdata->address = gnome_config_get_string("Address")) == NULL) {
                g_free(sdata);
                break;
            } 
            sdata->port = gnome_config_get_string_with_default("Port=119", NULL);
            sdata->name = g_strdup(sd[i]);
            props->servers = g_list_append(props->servers, sdata);
			gnome_config_pop_prefix();
        } 
    }
    g_strfreev(sd);
}

static void
save_servers(Props * props)
{
    server_data *sdata = NULL;
    GList *l = NULL;
    int i = 0;
    char *p = NULL;
	char **sd = NULL;
	char buf[256];
 	
    /* Out with the old */
    if ((p = gnome_config_get_string("/Pan/Servers/Names"))) {
        sd = g_strsplit(p, ",", -1);
        for (i = 0; sd[i]; i++) {
			g_snprintf(buf, 256, "/Pan/%s", sd[i]);
            gnome_config_clean_section(buf);
        }
        g_strfreev(sd);
    	g_free(p);
    }

    gnome_config_sync();

    /* In with the new */
    p = NULL;
	for (l = props->servers; l; l = l->next) {
        sdata = (server_data *) l->data;
		g_snprintf(buf, 256, "/Pan/%s/Address", sdata->name);
		gnome_config_set_string(buf, sdata->address);
		g_snprintf(buf, 256, "/Pan/%s/Port", sdata->name);
        gnome_config_set_string(buf, sdata->port);
        if (p) {
			p = g_realloc(p, strlen(p) + strlen(sdata->name) + 2);
			p = strcat(p, ",");
			p = strcat(p, sdata->name);
		} else {
			p = g_strdup(sdata->name);
		}
    }
    
	gnome_config_set_string("/Pan/Servers/Names", p);
    g_free(p);

	gnome_config_sync();
}

void
load_properties(Props * props)
{
    load_servers(props);

    props->headers = gnome_config_get_int_with_default("/Pan/Articles/Headers=2", NULL);
    props->expiration = gnome_config_get_int_with_default("/Pan/Articles/Expiration=30", NULL);
    if ((props->download_dir = gnome_config_get_string("/Pan/Paths/download_dir")) == NULL) {
		props->download_dir = gnome_util_prepend_user_home(".pan/download");
	}
    if ((props->temp_dir = gnome_config_get_string("/Pan/Paths/temp_dir")) == NULL) {
		props->temp_dir = gnome_util_prepend_user_home(".pan/tmp");
	}
    if ((props->data_dir = gnome_config_get_string("/Pan/Paths/data_dir")) == NULL) {
		props->data_dir = gnome_util_prepend_user_home(".pan/data");
	}
    props->full_name = gnome_config_get_string("/Pan/User/Full_Name");
    props->email = gnome_config_get_string("/Pan/User/Email");
    props->organization = gnome_config_get_string("/Pan/User/Organization");
    // Load other shit;
    directory_check(props->temp_dir);
    directory_check(props->data_dir);
    directory_check(props->download_dir);
    server_menu_update();
}

void
save_properties(Props * props)
{
    save_servers(props);

    gnome_config_set_int("/Pan/Articles/Headers", props->headers);
    gnome_config_set_int("/Pan/Articles/Expiration", props->expiration);
    gnome_config_set_string("/Pan/Binaries/download_dir", props->download_dir);
    gnome_config_set_string("/Pan/User/Full_Name",
                            gtk_entry_get_text(GTK_ENTRY(props_window->fullname_entry)));
    gnome_config_set_string("/Pan/User/Email",
                            gtk_entry_get_text(GTK_ENTRY(props_window->email_entry)));
    gnome_config_set_string("/Pan/User/Organization",
                            gtk_entry_get_text(GTK_ENTRY(props_window->organization_entry)));
    gnome_config_set_string("pan/Paths/download_dir",
                            gtk_entry_get_text(GTK_ENTRY(props_window->download_entry)));
    gnome_config_set_string("pan/Paths/temp_dir",
                            gtk_entry_get_text(GTK_ENTRY(props_window->temp_entry)));
    gnome_config_set_string("pan/Paths/data_dir",
                            gtk_entry_get_text(GTK_ENTRY(props_window->data_entry)));

    // Save other shit;

    gnome_config_sync();
}

static void
update_props_clist(void)
{
    gchar *text[2];
    GList *temp;
    server_data *data;
    gint row;

    gtk_clist_freeze(GTK_CLIST(props_window->server_clist));
    gtk_clist_clear(GTK_CLIST(props_window->server_clist));
    temp = new_props->servers;
    while (temp != NULL) {
        data = (server_data *) temp->data;
        text[0] = (data->name) ? data->name : g_strdup(_("Not Named"));
        text[1] = (data->address) ? data->address : g_strdup(_("No Address"));
        row = gtk_clist_prepend(GTK_CLIST(props_window->server_clist), text);
        gtk_clist_set_row_data(GTK_CLIST(props_window->server_clist), row, data);
        temp = temp->next;
    }
    gtk_clist_thaw(GTK_CLIST(props_window->server_clist));
}

static void
prop_apply(GtkWidget * box, int n)
{
    if (n != -1) {
        save_properties(new_props);
        load_properties(Pan.props);
        load_properties(new_props);
        update_props_clist();
        server_menu_update();        
    }
}

static void
edit_profile_dialog_clicked(GnomeDialog * dialog, gint button, server_data * data)
{

    switch (button) {
    case 0:     /* OK button */
        if (data == NULL) {
            data = (server_data *) g_malloc(sizeof(server_data));
            memset(data, 0, sizeof(server_data));
            new_props->servers = g_list_append(new_props->servers, data);
        }
        data->name = g_strdup(gtk_entry_get_text(GTK_ENTRY(props_window->entry1)));
        data->address = g_strdup(gtk_entry_get_text(GTK_ENTRY(props_window->entry2)));
        data->port = g_strdup(gtk_entry_get_text(GTK_ENTRY(props_window->entry3)));
        update_props_clist();
        gnome_property_box_changed(props_window->box);
        gtk_widget_destroy(GTK_WIDGET(dialog));
        break;
    default:    /* Cancel Button */
        gtk_widget_destroy(GTK_WIDGET(dialog));
        break;
    }
}

static void
edit_profile_changed(GnomeDialog * d)
{
    gnome_dialog_set_sensitive(d, 0, TRUE);
}

static void
edit_profile_dialog(server_data * data)
{
    GtkWidget *dialog;
    GtkWidget *table;
    GtkWidget *label1;
    GtkWidget *label2;
    GtkWidget *label3;
    GtkWidget *label4;

    dialog = gnome_dialog_new("New/Edit Server", GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL, NULL);

    gnome_dialog_set_sensitive(GNOME_DIALOG(dialog), 0, FALSE);

    gtk_signal_connect(GTK_OBJECT(dialog), "clicked", (GtkSignalFunc) edit_profile_dialog_clicked,
                       data);

    table = gtk_table_new(4, 2, FALSE);
    gtk_widget_show(table);

    gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), table);

    label1 = gtk_label_new("Enter the Following Information:");
    gtk_widget_show(label1);
    gtk_table_attach_defaults(GTK_TABLE(table), label1, 0, 2, 0, 1);

    label2 = gtk_label_new("Profile Name");
    gtk_widget_show(label2);
    gtk_table_attach(GTK_TABLE(table), label2, 0, 1, 1, 2,
                     GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

    props_window->entry1 = gtk_entry_new_with_max_length(32);
    if (data)
        gtk_entry_set_text(GTK_ENTRY(props_window->entry1), data->name);
    gtk_signal_connect_object(GTK_OBJECT(props_window->entry1), "changed",
                              GTK_SIGNAL_FUNC(edit_profile_changed), GTK_OBJECT(dialog));
    gtk_widget_show(props_window->entry1);
    gtk_table_attach(GTK_TABLE(table), props_window->entry1, 1, 2, 1, 2,
                     GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 20, 5);

    label3 = gtk_label_new("Server Address");
    gtk_widget_show(label3);
    gtk_table_attach(GTK_TABLE(table), label3, 0, 1, 2, 3,
                     GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

    props_window->entry2 = gtk_entry_new_with_max_length(32);
    if (data)
        gtk_entry_set_text(GTK_ENTRY(props_window->entry2), data->address);
    gtk_signal_connect_object(GTK_OBJECT(props_window->entry2), "changed",
                              GTK_SIGNAL_FUNC(edit_profile_changed), GTK_OBJECT(dialog));
    gtk_widget_show(props_window->entry2);
    gtk_table_attach(GTK_TABLE(table), props_window->entry2, 1, 2, 2, 3,
                     GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 20, 5);

    label4 = gtk_label_new("Server Port");
    gtk_widget_show(label4);
    gtk_table_attach(GTK_TABLE(table), label4, 0, 1, 3, 4,
                     GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

    props_window->entry3 = gtk_entry_new_with_max_length(32);
    gtk_entry_set_text(GTK_ENTRY(props_window->entry3), data ? data->port : "119");
    gtk_signal_connect_object(GTK_OBJECT(props_window->entry3), "changed",
                              GTK_SIGNAL_FUNC(edit_profile_changed), GTK_OBJECT(dialog));
    gtk_widget_show(props_window->entry3);
    gtk_table_attach(GTK_TABLE(table), props_window->entry3, 1, 2, 3, 4,
                     GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 20, 5);

    gtk_widget_show(dialog);
}

static void
server_clist_button_press(GtkWidget * widget, GdkEventButton * bevent)
{
    gint row;
    gint column;
    server_data *data;

    if (bevent->button != 1)
        gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");

    if ((bevent->button) == 1 && (bevent->type == GDK_2BUTTON_PRESS)) {
        gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");
        gtk_clist_get_selection_info(GTK_CLIST(widget), bevent->x, bevent->y,
                                     &row, &column);
        data = (server_data *) gtk_clist_get_row_data(GTK_CLIST(widget), row);
        edit_profile_dialog(data);
    }
}

static void
new_server(void)
{
    edit_profile_dialog(NULL);
}

static void
edit_server()
{
    GList *list;
    server_data *data;

    if ((list = GTK_CLIST(props_window->server_clist)->selection) == NULL)
        return;
    data = (server_data *) gtk_clist_get_row_data(GTK_CLIST(props_window->server_clist),
                                                  GPOINTER_TO_INT(list->data));
    edit_profile_dialog(data);
}

static void
delete_server()
{
    GList *list;
    server_data *data;

    if ((list = GTK_CLIST(props_window->server_clist)->selection) == NULL)
        return;
    data = (server_data *) gtk_clist_get_row_data(GTK_CLIST(props_window->server_clist),
                                                  GPOINTER_TO_INT(list->data));
    new_props->servers = g_list_remove(new_props->servers, data);
    g_free(data);

    update_props_clist();
    gnome_property_box_changed(props_window->box);
}

static void
cancel_properties()
{
    gtk_widget_destroy(GTK_WIDGET(props_window->box));
    g_free(new_props);
    g_free(props_window);
    props_window = NULL;
}

static GtkWidget *
paths_page(void)
{
    GtkWidget *table;
    GtkWidget *label;

    table = gtk_table_new(3, 2, FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(table), 2);
    gtk_table_set_col_spacings(GTK_TABLE(table), 2);

    label = gtk_label_new("Download Directory:");
    gtk_widget_show(label);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 4, 0);

    props_window->download_entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(props_window->download_entry), new_props->download_dir);
    gtk_signal_connect_object(GTK_OBJECT(props_window->download_entry), "changed",
                              GTK_SIGNAL_FUNC(gnome_property_box_changed),
                              GTK_OBJECT(props_window->box));
    gtk_widget_show(props_window->download_entry);
    gtk_table_attach(GTK_TABLE(table), props_window->download_entry, 1, 2, 0, 1,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

    label = gtk_label_new("Temporary Directory:");
    gtk_widget_show(label);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 4, 0);

    props_window->temp_entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(props_window->temp_entry), new_props->temp_dir);
    gtk_signal_connect_object(GTK_OBJECT(props_window->temp_entry), "changed",
                              GTK_SIGNAL_FUNC(gnome_property_box_changed),
                              GTK_OBJECT(props_window->box));
    gtk_widget_show(props_window->temp_entry);
    gtk_table_attach(GTK_TABLE(table), props_window->temp_entry, 1, 2, 1, 2,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

    label = gtk_label_new("Data Directory:");
    gtk_widget_show(label);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 4, 0);

    props_window->data_entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(props_window->data_entry), new_props->data_dir);
    gtk_signal_connect_object(GTK_OBJECT(props_window->data_entry), "changed",
                              GTK_SIGNAL_FUNC(gnome_property_box_changed),
                              GTK_OBJECT(props_window->box));
    gtk_widget_show(props_window->data_entry);
    gtk_table_attach(GTK_TABLE(table), props_window->data_entry, 1, 2, 2, 3,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

    return table;
}

static GtkWidget *
userinfo_page(void)
{
    GtkWidget *table;
    GtkWidget *label;

    table = gtk_table_new(3, 2, FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(table), 2);
    gtk_table_set_col_spacings(GTK_TABLE(table), 2);

    label = gtk_label_new("Full Name:");
    gtk_widget_show(label);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 4, 0);

    props_window->fullname_entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(props_window->fullname_entry), new_props->full_name);
    gtk_signal_connect_object(GTK_OBJECT(props_window->fullname_entry), "changed",
                              GTK_SIGNAL_FUNC(gnome_property_box_changed),
                              GTK_OBJECT(props_window->box));
    gtk_widget_show(props_window->fullname_entry);
    gtk_table_attach(GTK_TABLE(table), props_window->fullname_entry, 1, 2, 0, 1,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

    label = gtk_label_new("Email Address:");
    gtk_widget_show(label);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 4, 0);

    props_window->email_entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(props_window->email_entry), new_props->email);
    gtk_signal_connect_object(GTK_OBJECT(props_window->email_entry), "changed",
                              GTK_SIGNAL_FUNC(gnome_property_box_changed),
                              GTK_OBJECT(props_window->box));
    gtk_widget_show(props_window->email_entry);
    gtk_table_attach(GTK_TABLE(table), props_window->email_entry, 1, 2, 1, 2,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

    label = gtk_label_new("Organization:");
    gtk_widget_show(label);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 4, 0);

    props_window->organization_entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(props_window->organization_entry), new_props->organization);
    gtk_signal_connect_object(GTK_OBJECT(props_window->organization_entry), "changed",
                              GTK_SIGNAL_FUNC(gnome_property_box_changed),
                              GTK_OBJECT(props_window->box));
    gtk_widget_show(props_window->organization_entry);
    gtk_table_attach(GTK_TABLE(table), props_window->organization_entry, 1, 2, 2, 3,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

    return table;
}

static GtkWidget *
server_page(void)
{
    GtkWidget *vbox, *hbox, *button;

    static gchar *titles[] =
    {
        _("Name"),
        _("Address"),
    };

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox);

    props_window->server_clist = gtk_clist_new_with_titles(2, titles);
    gtk_clist_column_titles_passive(GTK_CLIST(props_window->server_clist));
    gtk_clist_set_selection_mode(GTK_CLIST(props_window->server_clist), GTK_SELECTION_SINGLE);

    gtk_clist_set_column_width(GTK_CLIST(props_window->server_clist), 0, 240);
    gtk_clist_set_column_width(GTK_CLIST(props_window->server_clist), 1, 40);

    gtk_widget_set_usize(props_window->server_clist, 280, 200);
    gtk_box_pack_start(GTK_BOX(hbox), props_window->server_clist, TRUE, TRUE, 5);
    gtk_signal_connect(GTK_OBJECT(props_window->server_clist), "button_press_event",
                       GTK_SIGNAL_FUNC(server_clist_button_press), NULL);

    gtk_widget_show(props_window->server_clist);
    update_props_clist();

    /* one button vbox */
    vbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
    gtk_widget_show(vbox);

    /* new server button */
    button = gtk_button_new_with_label("New...");
    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 10);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) new_server, NULL);
    gtk_widget_show(button);

    /* edit server button */
    button = gtk_button_new_with_label("Edit...");
    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 10);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) edit_server, NULL);
    gtk_widget_show(button);

    /* delete server button */
    button = gtk_button_new_with_label("Delete");
    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 10);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) delete_server, NULL);
    gtk_widget_show(button);    

    return(hbox);
}

void
properties_spawn(void)
{
    GtkWidget *w, *label;

    /* There can be only one! (properties window) */
    if (props_window) {
        gdk_window_raise(GTK_WIDGET(props_window->box)->window);
        return;
    }
    new_props = (Props *) g_malloc(sizeof(Props));
    memset(new_props, 0, sizeof(Props));
    new_props->servers = NULL;
    load_properties(new_props);

    props_window = (Props_window *) g_malloc(sizeof(Props_window));

    props_window->box = GNOME_PROPERTY_BOX(gnome_property_box_new());
    w = server_page();
    gtk_widget_show(w);
    label = gtk_label_new("Servers");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(props_window->box->notebook), w, label);
    w = userinfo_page();
    gtk_widget_show(w);
    label = gtk_label_new("User Info");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(props_window->box->notebook), w, label);
    w = paths_page();
    gtk_widget_show(w);
    label = gtk_label_new("Paths");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(props_window->box->notebook), w, label);
    gtk_signal_connect(GTK_OBJECT(props_window->box), "apply",
                       (GtkSignalFunc) prop_apply, NULL);
    gtk_signal_connect(GTK_OBJECT(props_window->box), "destroy",
                       (GtkSignalFunc) cancel_properties, NULL);
    gtk_widget_show(GTK_WIDGET(props_window->box));
}
