/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include "pan.h"

static GtkWidget *server_menu;
int menu_sensitized = TRUE;

static void
server_free(server_data *sdata)
{
	if (sdata) {
		g_free(sdata->address);
		g_free(sdata->port);
		g_free(sdata);
	}
}

void
server_menu_desensitize(void)
{
	
	menu_sensitized = FALSE;
    pan_lock(); 
	gtk_container_foreach(GTK_CONTAINER(server_menu), (GtkCallback)gtk_widget_set_sensitive, (void *)0);
    pan_unlock(); 
}

void
server_menu_sensitize(void)
{
    menu_sensitized = TRUE;
    pan_lock(); 
    gtk_container_foreach(GTK_CONTAINER(server_menu), (GtkCallback)gtk_widget_set_sensitive, (void *)1);
    pan_unlock(); 
}

static server_data *
server_fetch(gchar *name)
{
	server_data *sdata = NULL;
	char buf[256];
	
 	g_snprintf(buf, 256, "/Pan/%s/", name);
   	gnome_config_push_prefix(buf);
    
	if (gnome_config_has_section(buf)) {
		sdata = (server_data *) g_malloc(sizeof(server_data));
		memset(sdata, 0, sizeof(server_data));
		if ((sdata->address = gnome_config_get_string("Address")) == NULL) {
			g_free(sdata);
			sdata = NULL;
		} else {
			sdata->port = gnome_config_get_string_with_default("Port=119", NULL);
			sdata->name = g_strdup(name);
		}
	}
   	gnome_config_pop_prefix();
    
	return(sdata);
}

static void
server_set_active(char *name)
{
	char path[PATH_MAX];

	g_snprintf(path, PATH_MAX, "/%s/%s/", Pan.props->data_dir, name);
	directory_check(path);

	server_free(Pan.active_server); 
	Pan.active_server = server_fetch(name);
	article_ctree_clean();
	grouplist_free(Pan.grouplist);
	Pan.grouplist = NULL;
	group_clist_update();
	gnome_config_set_string("/Pan/Servers/Active", name);
	gnome_config_sync();
}

void
server_selected_cb (GtkWidget *menuitem, gchar *name)
{
	g_return_if_fail(name);
	
	if (Pan.active_server && !strcmp(name, Pan.active_server->name)) {
		return;
	} else {
		server_set_active(name);
    }
}                      

void
server_init(void)
{
	gchar *active; 

	if ((active = gnome_config_get_string("/Pan/Servers/Active")) == NULL) {
		Pan.active_server = NULL;
	} else {
        server_set_active(active);
		g_free(active);
	}
}

void
server_menu_create(GtkWidget *menu)
{
    server_menu = menu;
}

void
server_menu_update()
{
    GtkWidget *menuitem;
    GSList *group = NULL;
    gint i;
	gchar *p;
	static gchar **sd = NULL;
	
    gtk_container_foreach(GTK_CONTAINER(server_menu), (GtkCallback)gtk_widget_destroy, NULL);
    g_strfreev(sd);

    if ((p = gnome_config_get_string("/Pan/Servers/Names")) == NULL)
        return;
    
	sd = g_strsplit(p, ",", 0);
	g_free(p);
	
	g_return_if_fail(sd);

	p = gnome_config_get_string("/Pan/Servers/Active");

    for (i = 0; sd[i]; i++) {
        menuitem = gtk_radio_menu_item_new_with_label(group, sd[i]);
		group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM(menuitem));
        gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (menuitem), TRUE);
        gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(server_selected_cb), sd[i]);
        gtk_widget_show (menuitem);
        gtk_menu_append (GTK_MENU(server_menu), menuitem);
		if (p && (!strcmp(sd[i], p))) {
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), TRUE); 
		}
    }
	g_free(p);
    
	if (menu_sensitized == FALSE) {
   		server_menu_desensitize();
    }
}
