/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>
#include <gtk/gtk.h>
#include "pan.h"

static GtkWidget *text;
static article_data *article;

void
clear_text()
{
    gint i;

    i = gtk_text_get_length(GTK_TEXT(text));
    if (i > 0) {
	gtk_text_set_point(GTK_TEXT(text), i);
	gtk_text_backward_delete(GTK_TEXT(text), i);
    }
}

void
realize_text_cb(GtkWidget * text, gpointer data)
{
    clear_text();
    gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black,
		    NULL, "", -1);
}

void
update_text(article_data * adata)
{
    gchar *temp;

    pan_lock();
	if (article)
		article_free(article);
    article = article_copy(adata);

    clear_text();
    gtk_text_freeze(GTK_TEXT(text));
    switch (Pan.props->headers) {
    case 1:
	temp = g_strconcat("Subject:  ", article->subject, "\nAuthor:  ", article->author, "\nDate:  ", 
		ctime((time_t *) & article->date), "\n--\n", NULL);
	gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black,
			NULL, temp, strlen(temp));
	g_free(temp);
	break;
    case 2:
	gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black,
			NULL, article->header, strlen(article->header));
	gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black,
			NULL, "--\n", strlen("--\n"));
	break;
    default:
	break;
    }
    gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black,
		    NULL, article->body, strlen(article->body));
    gtk_text_thaw(GTK_TEXT(text));
    pan_unlock();
}

article_data *
displayed_article(void)
{
    return (article);
}

/* Create a scrolled text area that displays a "message" */
GtkWidget *
create_text(void)
{
    GtkWidget *table;
    GtkWidget *hscrollbar;
    GtkWidget *vscrollbar;

    /* Initialize Variables */
    article = NULL;
    text = NULL;

    /* Create a table to hold the text widget and scrollbars */
    table = gtk_table_new(2, 2, FALSE);

    /* Put a text widget in the upper left hand corner. Note the use of
     * GTK_SHRINK in the y direction */
    text = gtk_text_new(NULL, NULL);
    gtk_table_attach(GTK_TABLE(table), text, 0, 1, 0, 1,
		     GTK_FILL | GTK_EXPAND,
		     GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 0);
    gtk_widget_show(text);

    /* Put a HScrollbar in the lower left hand corner */
    hscrollbar = gtk_hscrollbar_new(GTK_TEXT(text)->hadj);
    gtk_table_attach(GTK_TABLE(table), hscrollbar, 0, 1, 1, 2,
		     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
    gtk_widget_show(hscrollbar);

    /* And a VScrollbar in the upper right */
    vscrollbar = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
    gtk_table_attach(GTK_TABLE(table), vscrollbar, 1, 2, 0, 1,
		     GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK,
		     0, 0);
    gtk_widget_show(vscrollbar);

    /* Add a handler to put a message in the text widget when it is realized */
    gtk_signal_connect(GTK_OBJECT(text), "realize",
		       GTK_SIGNAL_FUNC(realize_text_cb), NULL);

    return table;
}
